@@ -5,23 +5,23 @@ use std::{
 
 use num_enum::{IntoPrimitive, TryFromPrimitive, TryFromPrimitiveError};
 use thiserror::Error;
-use winapi::{
-    shared::winerror::ERROR_PARTIAL_COPY,
-    um::{
-        minwinbase::{
-            EXCEPTION_ACCESS_VIOLATION, EXCEPTION_ARRAY_BOUNDS_EXCEEDED, EXCEPTION_BREAKPOINT,
-            EXCEPTION_DATATYPE_MISALIGNMENT, EXCEPTION_FLT_DENORMAL_OPERAND,
-            EXCEPTION_FLT_DIVIDE_BY_ZERO, EXCEPTION_FLT_INEXACT_RESULT,
-            EXCEPTION_FLT_INVALID_OPERATION, EXCEPTION_FLT_OVERFLOW, EXCEPTION_FLT_STACK_CHECK,
-            EXCEPTION_FLT_UNDERFLOW, EXCEPTION_GUARD_PAGE, EXCEPTION_ILLEGAL_INSTRUCTION,
-            EXCEPTION_INT_DIVIDE_BY_ZERO, EXCEPTION_INT_OVERFLOW, EXCEPTION_INVALID_DISPOSITION,
-            EXCEPTION_INVALID_HANDLE, EXCEPTION_IN_PAGE_ERROR, EXCEPTION_NONCONTINUABLE_EXCEPTION,
-            EXCEPTION_PRIV_INSTRUCTION, EXCEPTION_SINGLE_STEP, EXCEPTION_STACK_OVERFLOW,
-        },
-        winnt::STATUS_UNWIND_CONSOLIDATE,
+use winapi::um::{
+    minwinbase::{
+        EXCEPTION_ACCESS_VIOLATION, EXCEPTION_ARRAY_BOUNDS_EXCEEDED, EXCEPTION_BREAKPOINT,
+        EXCEPTION_DATATYPE_MISALIGNMENT, EXCEPTION_FLT_DENORMAL_OPERAND,
+        EXCEPTION_FLT_DIVIDE_BY_ZERO, EXCEPTION_FLT_INEXACT_RESULT,
+        EXCEPTION_FLT_INVALID_OPERATION, EXCEPTION_FLT_OVERFLOW, EXCEPTION_FLT_STACK_CHECK,
+        EXCEPTION_FLT_UNDERFLOW, EXCEPTION_GUARD_PAGE, EXCEPTION_ILLEGAL_INSTRUCTION,
+        EXCEPTION_INT_DIVIDE_BY_ZERO, EXCEPTION_INT_OVERFLOW, EXCEPTION_INVALID_DISPOSITION,
+        EXCEPTION_INVALID_HANDLE, EXCEPTION_IN_PAGE_ERROR, EXCEPTION_NONCONTINUABLE_EXCEPTION,
+        EXCEPTION_PRIV_INSTRUCTION, EXCEPTION_SINGLE_STEP, EXCEPTION_STACK_OVERFLOW,
     },
+    winnt::STATUS_UNWIND_CONSOLIDATE,
 };
 
+#[cfg(feature = "syringe")]
+use winapi::shared::winerror::ERROR_PARTIAL_COPY;
+
 #[derive(Debug, Error)]
 /// Error enum representing either a windows api error or a nul error from an invalid interior nul.
 pub enum IoOrNulError {
@@ -49,57 +49,6 @@ pub enum GetLocalProcedureAddressError {
     UnsupportedRemoteTarget,
 }
 
-// TODO: add more specialized error variants
-/// Error enum for errors during syringe operations like injection, ejection or remote procedure calling.
-#[derive(Debug, Error)]
-pub enum SyringeError {
-    /// Variant representing an illegal interior nul value.
-    #[error("interior nul found")]
-    Nul(#[from] widestring::NulError<u16>),
-    /// Variant representing an io error.
-    #[error("io error: {}", _0)]
-    Io(io::Error),
-    /// Variant representing an unsupported target process.
-    #[error("unsupported target process")]
-    UnsupportedTarget,
-    /// Variant representing a windows api error inside the target process.
-    #[error("remote io error: {}", _0)]
-    RemoteIo(io::Error),
-    /// Variant representing an unhandled exception inside the target process.
-    #[error("remote exception: {}", _0)]
-    RemoteException(ExceptionCode),
-    /// Variant representing an inaccessible target process.
-    /// This can occur if it crashed or was terminated.
-    #[error("inaccessible target process")]
-    ProcessInaccessible,
-    /// Variant representing an error while loading an pe file.
-    #[cfg(target_arch = "x86_64")]
-    #[cfg(feature = "into_x86_from_x64")]
-    #[error("failed to load pe file: {}", _0)]
-    Goblin(#[from] goblin::error::Error),
-}
-
-impl From<io::Error> for SyringeError {
-    fn from(err: io::Error) -> Self {
-        if err.raw_os_error() == Some(ERROR_PARTIAL_COPY as _)
-            || err.kind() == io::ErrorKind::PermissionDenied
-        {
-            Self::ProcessInaccessible
-        } else {
-            Self::Io(err)
-        }
-    }
-}
-
-impl From<IoOrNulError> for SyringeError {
-    fn from(err: IoOrNulError) -> Self {
-        match err {
-            IoOrNulError::Nul(e) => e.into(),
-            IoOrNulError::Io(e) => e.into(),
-        }
-    }
-}
-
 #[derive(
     Debug, TryFromPrimitive, IntoPrimitive, Clone, Copy, Eq, PartialEq, PartialOrd, Ord, Hash,
 )]
@@ -205,3 +154,89 @@ impl Display for ExceptionCode {
         }
     }
 }
+
+#[derive(Debug, Error)]
+/// An error representing either an unhandled exception or an io error.
+pub enum ExceptionOrIoError {
+    /// Variant representing an io error.
+    #[error("remote io error: {}", _0)]
+    Io(io::Error),
+    /// Variant representing an unhandled exception.
+    #[error("remote exception: {}", _0)]
+    Exception(ExceptionCode),
+}
+
+/// Error enum for errors during syringe operations like injection and ejection.
+#[derive(Debug, Error)]
+#[cfg(feature = "syringe")]
+#[cfg_attr(feature = "doc-cfg", doc(cfg(feature = "syringe")))]
+pub enum SyringeError {
+    /// Variant representing an illegal interior nul value.
+    #[error("interior nul found")]
+    Nul(#[from] widestring::NulError<u16>),
+    /// Variant representing an io error.
+    #[error("io error: {}", _0)]
+    Io(io::Error),
+    /// Variant representing an unsupported target process.
+    #[error("unsupported target process")]
+    UnsupportedTarget,
+    /// Variant representing an io error inside the target process.
+    #[error("remote io error: {}", _0)]
+    RemoteIo(io::Error),
+    /// Variant representing an unhandled exception inside the target process.
+    #[error("remote exception: {}", _0)]
+    RemoteException(ExceptionCode),
+    /// Variant representing an inaccessible target process.
+    /// This can occur if it crashed or was terminated.
+    #[error("inaccessible target process")]
+    ProcessInaccessible,
+    /// Variant representing an inaccessible target module.
+    /// This can occur if the target module was ejected or unloaded.
+    #[error("inaccessible target module")]
+    ModuleInaccessible,
+    /// Variant representing an error while loading an pe file.
+    #[cfg(target_arch = "x86_64")]
+    #[cfg(feature = "into-x86-from-x64")]
+    #[error("failed to load pe file: {}", _0)]
+    Goblin(#[from] goblin::error::Error),
+}
+
+#[cfg(feature = "syringe")]
+impl From<io::Error> for SyringeError {
+    fn from(err: io::Error) -> Self {
+        if err.raw_os_error() == Some(ERROR_PARTIAL_COPY as _)
+            || err.kind() == io::ErrorKind::PermissionDenied
+        {
+            Self::ProcessInaccessible
+        } else {
+            Self::Io(err)
+        }
+    }
+}
+
+#[cfg(feature = "syringe")]
+impl From<ExceptionCode> for SyringeError {
+    fn from(err: ExceptionCode) -> Self {
+        Self::RemoteException(err)
+    }
+}
+
+#[cfg(feature = "syringe")]
+impl From<IoOrNulError> for SyringeError {
+    fn from(err: IoOrNulError) -> Self {
+        match err {
+            IoOrNulError::Nul(e) => e.into(),
+            IoOrNulError::Io(e) => e.into(),
+        }
+    }
+}
+
+#[cfg(feature = "syringe")]
+impl From<ExceptionOrIoError> for SyringeError {
+    fn from(err: ExceptionOrIoError) -> Self {
+        match err {
+            ExceptionOrIoError::Io(e) => Self::RemoteIo(e),
+            ExceptionOrIoError::Exception(e) => Self::RemoteException(e),
+        }
+    }
+}
