@@ -2,7 +2,7 @@ use cstr::cstr;
 use iced_x86::{code_asm::*, IcedError};
 use num_enum::TryFromPrimitive;
 use path_absolutize::Absolutize;
-use std::{ffi::c_void, io, lazy::OnceCell, mem, path::Path};
+use std::{io, lazy::OnceCell, mem, path::Path};
 use u16cstr::u16cstr;
 use widestring::U16CString;
 use winapi::shared::{
@@ -11,14 +11,14 @@ use winapi::shared::{
 };
 
 use crate::{
-    error::{ExceptionCode, SyringeError},
+    error::{ExceptionCode, ExceptionOrIoError, SyringeError},
     process::{
-        memory::{RemoteBox, RemoteBoxAllocator},
+        memory::{RemoteAllocation, RemoteBox, RemoteBoxAllocator},
         BorrowedProcess, BorrowedProcessModule, ModuleHandle, OwnedProcess, Process, ProcessModule,
     },
 };
 
-#[cfg(all(target_arch = "x86_64", feature = "into_x86_from_x64"))]
+#[cfg(all(target_arch = "x86_64", feature = "into-x86-from-x64"))]
 use {
     goblin::pe::PE,
     std::{convert::TryInto, fs, mem::MaybeUninit, path::PathBuf, time::Duration},
@@ -26,14 +26,14 @@ use {
     winapi::{shared::minwindef::MAX_PATH, um::wow64apiset::GetSystemWow64DirectoryW},
 };
 
-#[cfg(feature = "remote_procedure")]
+#[cfg(feature = "rpc-core")]
 use winapi::shared::{minwindef::FARPROC, ntdef::LPCSTR};
 
 type LoadLibraryWFn = unsafe extern "system" fn(LPCWSTR) -> HMODULE;
 type FreeLibraryFn = unsafe extern "system" fn(HMODULE) -> BOOL;
 type GetLastErrorFn = unsafe extern "system" fn() -> DWORD;
-#[cfg(feature = "remote_procedure")]
-type GetProcAddressFn = unsafe extern "system" fn(HMODULE, LPCSTR) -> FARPROC;
+#[cfg(feature = "rpc-core")]
+pub(crate) type GetProcAddressFn = unsafe extern "system" fn(HMODULE, LPCSTR) -> FARPROC;
 
 #[derive(Debug, Clone)]
 pub(crate) struct InjectHelpData {
@@ -41,7 +41,7 @@ pub(crate) struct InjectHelpData {
     load_library_offset: usize,
     free_library_offset: usize,
     get_last_error_offset: usize,
-    #[cfg(feature = "remote_procedure")]
+    #[cfg(feature = "rpc-core")]
     get_proc_address_offset: usize,
 }
 
@@ -57,7 +57,7 @@ impl InjectHelpData {
     pub fn get_get_last_error(&self) -> GetLastErrorFn {
         unsafe { mem::transmute(self.kernel32_module as usize + self.get_last_error_offset) }
     }
-    #[cfg(feature = "remote_procedure")]
+    #[cfg(feature = "rpc-core")]
     pub fn get_proc_address_fn_ptr(&self) -> GetProcAddressFn {
         unsafe { mem::transmute(self.kernel32_module as usize + self.get_proc_address_offset) }
     }
@@ -84,13 +84,14 @@ impl InjectHelpData {
 /// syringe.eject(injected_payload).unwrap();
 /// ```
 #[derive(Debug)]
+#[cfg_attr(feature = "doc-cfg", doc(cfg(feature = "syringe")))]
 pub struct Syringe {
     pub(crate) inject_help_data: OnceCell<InjectHelpData>,
     pub(crate) remote_allocator: RemoteBoxAllocator,
     load_library_w_stub: OnceCell<LoadLibraryWStub>,
-    #[cfg(feature = "remote_procedure")]
+    #[cfg(feature = "rpc-core")]
     pub(crate) get_proc_address_stub:
-        OnceCell<crate::RemoteProcedureStub<crate::GetProcAddressParams, FARPROC>>,
+        OnceCell<crate::rpc::RemoteProcedureStub<crate::rpc::GetProcAddressParams, FARPROC>>,
 }
 
 impl Syringe {
@@ -101,7 +102,7 @@ impl Syringe {
             remote_allocator: RemoteBoxAllocator::new(process),
             inject_help_data: OnceCell::new(),
             load_library_w_stub: OnceCell::new(),
-            #[cfg(feature = "remote_procedure")]
+            #[cfg(feature = "rpc-core")]
             get_proc_address_stub: OnceCell::new(),
         }
     }
@@ -133,7 +134,7 @@ impl Syringe {
             U16CString::from_os_str(module_path.as_os_str())?.into_vec_with_nul();
         let remote_wide_module_path = self
             .remote_allocator
-            .alloc_and_copy(wide_module_path.as_slice())?;
+            .alloc_and_copy_buf(wide_module_path.as_slice())?;
 
         let injected_module_handle =
             load_library_w.call(remote_wide_module_path.as_raw_ptr().cast())?;
@@ -217,37 +218,34 @@ impl Syringe {
 
         match (is_target_x64, is_self_x64) {
             (true, true) | (false, false) => Self::load_inject_help_data_for_current_target(),
-            #[cfg(all(target_arch = "x86_64", feature = "into_x86_from_x64"))]
+            #[cfg(all(target_arch = "x86_64", feature = "into-x86-from-x64"))]
             (false, true) => Self::_load_inject_help_data_for_process(process),
             _ => Err(SyringeError::UnsupportedTarget),
         }
     }
 
     #[allow(dead_code)]
-    pub(crate) fn remote_exit_code_to_exception(exit_code: u32) -> Result<(), SyringeError> {
+    pub(crate) fn remote_exit_code_to_exception(exit_code: u32) -> Result<u32, ExceptionCode> {
         if exit_code == 0 {
-            return Ok(());
+            return Ok(exit_code);
         }
 
         match ExceptionCode::try_from_primitive(exit_code) {
-            Ok(exception) => Err(SyringeError::RemoteException(exception)),
-            Err(_) => Err(SyringeError::RemoteIo(io::Error::new(
-                io::ErrorKind::Other,
-                "unknown remote process error",
-            ))),
+            Ok(exception) => Err(exception),
+            Err(_) => Ok(exit_code),
         }
     }
 
     pub(crate) fn remote_exit_code_to_error_or_exception(
         exit_code: u32,
-    ) -> Result<(), SyringeError> {
+    ) -> Result<(), ExceptionOrIoError> {
         if exit_code == 0 {
             return Ok(());
         }
 
         match ExceptionCode::try_from_primitive(exit_code) {
-            Ok(exception) => Err(SyringeError::RemoteException(exception)),
-            Err(_) => Err(SyringeError::RemoteIo(io::Error::from_raw_os_error(
+            Ok(exception) => Err(ExceptionOrIoError::Exception(exception)),
+            Err(_) => Err(ExceptionOrIoError::Io(io::Error::from_raw_os_error(
                 exit_code as _,
             ))),
         }
@@ -264,7 +262,7 @@ impl Syringe {
             kernel32_module.get_local_procedure_address_cstr(cstr!("FreeLibrary"))?;
         let get_last_error_fn_ptr =
             kernel32_module.get_local_procedure_address_cstr(cstr!("GetLastError"))?;
-        #[cfg(feature = "remote_procedure")]
+        #[cfg(feature = "rpc-core")]
         let get_proc_address_fn_ptr =
             kernel32_module.get_local_procedure_address_cstr(cstr!("GetProcAddress"))?;
 
@@ -274,14 +272,14 @@ impl Syringe {
             free_library_offset: free_library_fn_ptr as usize - kernel32_module.handle() as usize,
             get_last_error_offset: get_last_error_fn_ptr as usize
                 - kernel32_module.handle() as usize,
-            #[cfg(feature = "remote_procedure")]
+            #[cfg(feature = "rpc-core")]
             get_proc_address_offset: get_proc_address_fn_ptr as usize
                 - kernel32_module.handle() as usize,
         })
     }
 
     #[cfg(target_arch = "x86_64")]
-    #[cfg(feature = "into_x86_from_x64")]
+    #[cfg(feature = "into-x86-from-x64")]
     fn _load_inject_help_data_for_process(
         process: BorrowedProcess<'_>,
     ) -> Result<InjectHelpData, SyringeError> {
@@ -321,7 +319,7 @@ impl Syringe {
             .find(|export| matches!(export.name, Some("GetLastError")))
             .unwrap();
 
-        #[cfg(feature = "remote_procedure")]
+        #[cfg(feature = "rpc-core")]
         let get_proc_address_export = pe
             .exports
             .iter()
@@ -333,12 +331,12 @@ impl Syringe {
             load_library_offset: load_library_export.rva,
             free_library_offset: free_library_export.rva,
             get_last_error_offset: get_last_error_export.rva,
-            #[cfg(feature = "remote_procedure")]
+            #[cfg(feature = "rpc-core")]
             get_proc_address_offset: get_proc_address_export.rva,
         })
     }
 
-    #[cfg(all(target_arch = "x86_64", feature = "into_x86_from_x64"))]
+    #[cfg(all(target_arch = "x86_64", feature = "into-x86-from-x64"))]
     fn wow64_dir() -> Result<PathBuf, io::Error> {
         let mut path_buf = MaybeUninit::uninit_array::<MAX_PATH>();
         let path_buf_len: u32 = path_buf.len().try_into().unwrap();
@@ -355,7 +353,7 @@ impl Syringe {
 
 #[derive(Debug)]
 struct LoadLibraryWStub {
-    code: RemoteBox<[u8]>,
+    code: RemoteAllocation,
     result: RemoteBox<ModuleHandle>,
 }
 
@@ -368,20 +366,20 @@ impl LoadLibraryWStub {
 
         let code = if remote_allocator.process().is_x86()? {
             Self::build_code_x86(
-                inject_data.get_load_library_fn_ptr() as *const _,
+                inject_data.get_load_library_fn_ptr(),
                 result.as_raw_ptr().cast(),
-                inject_data.get_get_last_error() as *const _,
+                inject_data.get_get_last_error(),
             )
             .unwrap()
         } else {
             Self::build_code_x64(
-                inject_data.get_load_library_fn_ptr() as *const _,
+                inject_data.get_load_library_fn_ptr(),
                 result.as_raw_ptr().cast(),
-                inject_data.get_get_last_error() as *const _,
+                inject_data.get_get_last_error(),
             )
             .unwrap()
         };
-        let code = remote_allocator.alloc_and_copy(code.as_slice())?;
+        let code = remote_allocator.alloc_and_copy_buf(code.as_slice())?;
 
         Ok(Self { code, result })
     }
@@ -406,14 +404,13 @@ impl LoadLibraryWStub {
         self.code.process()
     }
 
+    #[allow(clippy::fn_to_numeric_cast, clippy::fn_to_numeric_cast_with_truncation)]
     fn build_code_x86(
-        load_library_w: *const c_void,
-        return_buffer: *mut c_void,
-        get_last_error: *const c_void,
+        load_library_w: LoadLibraryWFn,
+        return_buffer: *mut HMODULE,
+        get_last_error: GetLastErrorFn,
     ) -> Result<Vec<u8>, IcedError> {
-        assert!(!load_library_w.is_null());
         assert!(!return_buffer.is_null());
-        assert!(!get_last_error.is_null());
         assert_eq!(load_library_w as u32 as usize, load_library_w as usize);
         assert_eq!(return_buffer as u32 as usize, return_buffer as usize);
         assert_eq!(get_last_error as u32 as usize, get_last_error as usize);
@@ -445,14 +442,13 @@ impl LoadLibraryWStub {
         Ok(code)
     }
 
+    #[allow(clippy::fn_to_numeric_cast, clippy::fn_to_numeric_cast_with_truncation)]
     fn build_code_x64(
-        load_library_w: *const c_void,
-        return_buffer: *mut c_void,
-        get_last_error: *const c_void,
+        load_library_w: LoadLibraryWFn,
+        return_buffer: *mut HMODULE,
+        get_last_error: GetLastErrorFn,
     ) -> Result<Vec<u8>, IcedError> {
-        assert!(!load_library_w.is_null());
         assert!(!return_buffer.is_null());
-        assert!(!get_last_error.is_null());
 
         let mut asm = CodeAssembler::new(64)?;
 
